#! /usr/bin/env python
'''
:copyright: (c) 2014 by abusix GmbH
:license: Apache2, see LICENSE.txt for more details.
'''
from __future__ import print_function

import logging

from sys import exit, argv
from argparse import ArgumentParser, FileType, SUPPRESS
from pyxarf import Xarf
from pyxarf.exceptions import MissingParameterError, ValidationError
from xarfmail import SMTP, XarfMail, lookup_contact
import yaml

class Xarfutil(object):
    '''
    helper class for interaction with pyxarf


    '''
    def __init__(self):
        '''
        handles command line parser initialization


        '''
        self.xarf_args = {}
        self._groups = {}

        self._parser = ArgumentParser(
            description='xarfutil - xarf command line utility',
            argument_default=SUPPRESS,
        )
        self._add_arguments(self._parser)

        (self.args, self._extra_args) = self._parser.parse_known_args()
        self.args = vars(self.args)

        if 'args' in self.args: # move args= to extra_args
            self._extra_args.insert(1, self.args.pop('args')[0])

        # make sure parameter mode is not used together with file mode
        self._handle_arg_conflicts(
            ignore_groups=('mail options','output options')
        )

        self._check_file_parameters()

        # file mode was used
        if 'machine_readable' in self.xarf_args:
            self.xarf = Xarf.from_machine_readable
            return
        else: # parameter mode used
            self.xarf = Xarf

        for parameter in self._groups['parameter mode']:
            if parameter in self.args:
                self.xarf_args[parameter] = self.args[parameter]

        # convert foo=bar, bleh=blub to dict
        self._extra_args_to_dict()
        # .args contains all args
        self.args.update(self._extra_args)
        # .xarf_args contains only xarf related args
        self.xarf_args.update(self._extra_args)

    def _add_arguments(self, parser):
        '''
        configures the argument parser for handling
        all command line options

        :param parser: handle to ArgumentParser
        :type parser: :py:class:`ArgumentParser`:

        '''
        parser.add_argument('--debug', action='store_true',
            help='enable debug mode',
        )
        parser.add_argument('--send-email', action='store_true',
            help='generate and send report by mail',
        )
        parser.add_argument('--lookup-contact', action='store_true',
            help='automatically lookup abuse contact with querycontacts'
        )

        group_1 = parser.add_argument_group(
            'parameter mode',
            'read report data from parameters (conflicts with file mode)'
        )
        self._add_group_argument(group_1, '--greeting',
            type=str, metavar='<string>',
            help='greeting text (xarf part 1, only for mail generation)'
        )
        self._add_group_argument(group_1, '--evidence',
            type=str, metavar='<string>',
            help='evidence text (xarf part 2)'
        )
        self._add_group_argument(group_1, '--schema-url',
            type=str, metavar='<url>',
            help='url of json schema'
        )
        self._add_group_argument(group_1, '--schema-cache',
            type=str, metavar='<path>',
            help='path for caching schemas'
        )
        self._add_group_argument(group_1, '--reported-from',
            type=str, metavar='<email>',
            help='from email address of reporter'
        )
        self._add_group_argument(group_1, '--category',
            type=str, metavar='<string>',
            help='report category'
        )
        self._add_group_argument(group_1, '--report-type',
            type=str, metavar='<string>',
            help='report type'
        )
        self._add_group_argument(group_1, '--report-id',
            type=str, metavar='<string>',
            help='report id'
        )
        self._add_group_argument(group_1, '--date',
            type=str, metavar='<timestamp>',
            help='date of report'
        )
        self._add_group_argument(group_1, '--source',
            type=str, metavar='<ip>',
            help='source of report'
        )
        self._add_group_argument(group_1, '--source-type',
            type=str, metavar='<string>',
            help='type of source'
        )
        self._add_group_argument(group_1, '--attachment',
            type=str, metavar='<mime-type>',
            help='attachment mime-type'
        )
        self._add_group_argument(group_1, 'args', nargs='*',
            help='additional arguments, dependent on json schema'
        )

        group_2 = parser.add_argument_group(
            'file mode',
            'read report data from input files (conflicts with parameter mode)'
        )
        self._add_group_argument(group_2, '--file-evidence',
            type=FileType('r'), metavar='<path>',
            help='file with evidence data (xarf part 3)'
        )
        self._add_group_argument(group_2, '--file-machine-readable',
            type=FileType('r'), metavar='<path>',
            help='file with machine readable data (xarf part 2)'
        )
        self._add_group_argument(group_2, '--file-greeting',
            type=FileType('r'), metavar='<path>',
            help='file with greeting text (xarf part 1)'
        )

        group_3 = parser.add_argument_group(
            'output options',
            'various settings for printing validated data to stdout'
        )
        self._add_group_argument(group_3, '--output-json',
            action='store_true',
            help='print validated xarf report in json format'
        )
        self._add_group_argument(group_3, '--output-yaml',
            action='store_true',
            help='print validated xarf report in yaml format'
        )
        self._add_group_argument(group_3, '--output-email',
            action='store_true',
            help='print validated xarf report as raw email'
        )

        group_4 = parser.add_argument_group(
            'mail options', 'options when directly sending reports by email'
        )
        self._add_group_argument(group_4, '--mail-server-host',
            type=str, metavar='<hostname>',
            help='mail server ip or hostname'
        )
        self._add_group_argument(group_4, '--mail-server-port',
            type=str, metavar='<port>',
            help='mail server port'
        )
        self._add_group_argument(group_4, '--mail-server-user',
            type=str, metavar='<username>',
            help='mail server username'
        )
        self._add_group_argument(group_4, '--mail-server-pass',
            type=str, metavar='<password>',
            help='mail server password'
        )
        self._add_group_argument(group_4, '--mail-from',
            type=str, metavar='<from>',
            help='sender of report'
        )
        self._add_group_argument(group_4, '--mail-subject',
            type=str, metavar='<subject>',
            help='subject of report'
        )
        self._add_group_argument(group_4, '--mail-to',
            type=str, metavar='<recipient>',
            help='recipient of report'
        )

    def _check_file_parameters(self):
        '''
        checks if file mode command line arguments are used and reads
        the specified files to the corresponding variables

        '''
        file_handles = self._get_settings('file mode')

        if any(f in self.args for f in file_handles):
            if 'file_machine_readable' not in self.args:
                exit(
                    'error: --file-machine-readable is required with file mode'
                )

            if 'file_greeting' in self.args:
                self.args['greeting'] = file_handles[
                    'file_greeting'].read()

            if 'file_evidence' in self.args:
                self.xarf_args['evidence'] = file_handles[
                    'file_evidence'].read()

            try:
                self.xarf_args['machine_readable'] = yaml.load(
                    file_handles['file_machine_readable']
                )
            except yaml.scanner.ScannerError as e:
                exit('error: --file-machine-readable does not specify a valid '
                     'json or yaml file:\n%s' % e)

    def _add_group_argument(self, group, *args, **kwargs):
        '''
        wrapper function for :py:func:`add_argument` of
        ArgumentParser for saving group assignment in self._groups

        :param group: handle of group object
        :type group: py:func:`ArgumentParser` group object
        :param args: args of add_argument call
        :type args: list
        :param kwargs: kwargs of add_argument call
        :type kwargs: dict

        '''
        title = group.title

        arg = args[0].lstrip('-').replace('-', '_')
        if arg != 'args':
            if title not in self._groups:
                self._groups[title] = [arg]
            else:
                self._groups[title].append(arg)

        group.add_argument(*args, **kwargs)

    def _extra_args_to_dict(self):
        '''
        converts foo=bar style command line arguments to a dict

        '''
        extra_args = {}

        for i in range(0, len(self._extra_args), 2):
            key = self._extra_args[i].lstrip('-')
            try:
                value = self._cast_value(self._extra_args[i+1])
            except IndexError: # invalid parameter gets dropped
                continue
            extra_args[key] = value
        self._extra_args = extra_args

    def _cast_value(self, value):
        '''
        command line parameters are always strings, but some xarf
        schemas require int or float type arguments. this function
        tries to cast every command line value to it's real type

        :param value: value to cast
        :type value: object

        :returns: value as int, if it's an integer
        :rtype: int
        :returns: value as float, if it's a float
        :rtype: float
        :returns: value as int, if it's an integer
        :rtype: int

        '''
        try:
            if int(value) == float(value):
                return int(value)
        except:
            try:
                return float(value)
            except:
                return str(value)

    def _handle_arg_conflicts(self, ignore_groups):
        '''
        makes sure that all command line argument groups which are not
        passed to this function via the ignore_groups parameter conflict

        :param ignore_groups: groups which are allowed to conflict
        :type ignore_groups: tuple

        '''
        group_match = None
        for group in self._groups:
            if group in ignore_groups:
                continue
            for arg in self._groups[group]:
                arg = arg.replace('-', '_')
                if arg in self.args:
                    if not group_match:
                        group_match = group
                    elif group != group_match:
                        exit(
                            'error: parameters from group "%s" conflict with '
                            'group "%s"!' % (group, group_match)
                        )

    def _get_settings(self, group):
        '''
        returns all settings of the specified argument group

        :param group: name of argument group
        :type group: str

        :returns: settings
        :rtype: dict

        '''
        settings = {}
        if group in self._groups:
            for setting in self._groups[group]:
                if setting in self.args:
                    settings[setting] = self.args[setting]
        return settings

    def get_mail_to(self):
        mail_settings = self.get_mail_settings()
        mail_to = None
        if 'mail_to' in mail_settings:
            mail_to = mail_settings['mail_to']
        if 'lookup_contact' in util.args and \
                report.machine_readable['Source-Type'][:2] == 'ip':
            try:
                mail_to = lookup_contact(report.machine_readable['Source'])
            except ImportError:
                raise
            except:
                pass
        if not mail_to:
            exit("You have to supply a to mail address either by using --mail-to or --lookup-contact")
        return mail_to

    def to_mail(self, report, mail_to=None):
        '''
        returns the raw email of the specified xarf report_obj

        :param report: xarf report object
        :type report: :py:class:`Xarf`: object

        :returns: raw mail representation of xarf report
        :rtype: str

        '''
        greeting = ''

        if not mail_to:
            mail_to = self.get_mail_to()

        mail_settings = self.get_mail_settings()
        mail_from = mail_settings['mail_from']
        subject = mail_settings['mail_subject']

        if 'greeting' in util.args:
            greeting = util.args['greeting']


        self.mail_obj = XarfMail(report, mail_from, mail_to, subject, greeting)
        return str(self.mail_obj)

    def get_report(self, report):
        '''
        returns the report in the format specified with the --output-*
        command line switch

        :param report: xarf report object
        :type report: :py:class:`Xarf`: object

        :returns: json representation of xarf report
        :rtype: str
        :returns: yaml representation of xarf report
        :rtype: str
        :returns: raw mail representation of xarf report
        :rtype: str

        :raises: :py:class:`ValidationError`: if schema validation failed

        '''
        formats = {
            'output_json': (report.to_json, {}),
            'output_yaml': (report.to_yaml, {}),
            'output_email': (self.to_mail, {'report': report}),
        }
        try:
            for format in formats:
                if format in self.args:
                    func, args = formats[format]
                    return func(**args)
            else:
                return report.to_json()
        except ValidationError as e:
            exit('error: validation failed! reason(s):\n%s' % e)

    def print_help(self):
        '''
        shows the dynamically built help of :py:class:`ArgumentParser`:

        '''
        if self._parser:
            self._parser.print_help()

    def get_mail_settings(self):
        '''
        returns all mail option settings

        :returns: output of :py:func:`_get_settings`:
        :rtype: dict

        '''
        return self._get_settings('mail options')

    def get_file_handles(self):
        '''
        returns all file mode settings

        :returns: output of :py:func:`_get_settings`:
        :rtype: dict

        '''
        return self._get_settings('file mode')

if __name__ == '__main__':
    '''
    main function with primary application logic and error handling.
    uses :py:class:`Xarfutil`: helper class for wrapping all calls
    to :py:class:`pyxarf.Xarf`:

    '''
    util = Xarfutil()

    if len(argv) == 1:
        exit(util.print_help())

    try:
        report = util.xarf(**util.xarf_args)
    except MissingParameterError as e:
        exit('error: %s\nadd missing parameter(s) with --%s'
            % (e, ' --'.join(e.missing_parameters).replace('_', '-'))
        )
    except ValidationError as e:
        exit('error: validation failed! reason(s):\n%s' % e)

    if 'send_email' in util.args:
        mail_to = util.get_mail_to()
        raw_email = util.to_mail(report, mail_to)

        mail_settings = util.get_mail_settings()
        mail_from = mail_settings['mail_from']
        username = None
        password = None

        if 'mail_server_user' in mail_settings and\
                'mail_server_pass' in mail_settings:
            username = mail_settings['mail_server_user']
            password = mail_settings['mail_server_pass']

        smtp_server = SMTP(
            mail_settings['mail_server_host'],
            mail_settings['mail_server_port'],
            username,
            password,
        )

        try:
            smtp_server.send(mail_from, mail_to, raw_email)
        except:
            exit('Something went wrong while sending the Report.')
        else:
            exit('Report sent.')
    else:
        print(util.get_report(report))




import smtplib
from email import charset
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText


charset.add_charset('utf-8', charset.SHORTEST, charset.QP)

try:
    from querycontacts import ContactFinder
    cf = ContactFinder()
except ImportError as exception:
    cf = exception


def lookup_contact(ip):
    '''
    uses querycontacts if available to lookup abuse contact(s) for given ip

    :param ip: ip to lookup abuse contact for
    :type ip: str

    :returns: list of contacts or None
    :rtype: list
    :rtype: none

    :raises: :py:class:`ImportError`

    '''
    if isinstance(cf, ImportError):
        raise cf
    else:
        return cf.find(ip)


class SMTPException(Exception): pass


class SMTP(object):

    '''
    Class for sending E-Mails via SMTP

    :param host: Mail server hostname or ip address
    :type host: str
    :param port: Port of mail server (default: 25)
    :type mail_server_port: int
    :param user: Username if SMTP auth is required
    :type user: str
    :param password: Password if SMTP auth is required
    :type password: str
    '''

    def __init__(self, host, port=25, user=None, password=None):
        self._host = host
        self._port = port
        self._user = user
        self._password = password

    def send(self, mail_from, mail_to, email):
        '''
        Send the E-Mail via SMTP to the specified mail server.
        SMTP auth supported.

        :returns: True if mail was sent successfully
        :rtype: bool

        :raises: :py:class:`SMTPException`

        '''
        smtp = smtplib.SMTP(self._host, self._port)

        if self._user and self._password:
            smtp.login(self._user, self._password)

        ret = smtp.sendmail(mail_from, mail_to, email)
        smtp.quit()

        if len(ret):
            raise SMTPException(ret)

        return True


class MultiPartMail(object):
    '''
    Base class for multipart E-Mails

    :param mail_from: Sender E-Mail address
    :type mail_from: str
    :param subject: Subject of Network Abuse Report
    :type subject: str
    :param mail_to: Recipient E-Mail address can be provided as list
        for addressing multiple recipients
    :type mail_to: str or list
    '''
    def __init__(self, mail_to, mail_from, subject, subtype):
        self._email = MIMEMultipart(subtype)
        self._email['From'] = mail_from
        self._email['Subject'] = subject

        if type(mail_to) is list:
            self._email['To'] = ','.join(mail_to)
        else:
            self._email['To'] = mail_to


class XarfMail(MultiPartMail):
    '''
    Helper class for building an X-ARF E-Mail report

    :param xarf: X-ARF object to report
    :type xarf: :py:class:`Xarf`:
    :param mail_from: Sender E-Mail address
    :type mail_from: str
    :param mail_to: Recipient E-Mail address can be provided as list
        for addressing multiple recipients
    :type mail_to: str or list
    :param subject: Subject of Network Abuse Report
    :type subject: str
    :param greeting: Human readable text which contains at least basic
        information about the reported incidenteeting text in the first
        MIME Part
    :type greeting: unicode
    '''
    def __init__(self, xarf, mail_from, mail_to, subject, greeting):
        MultiPartMail.__init__(self, mail_to, mail_from, subject, 'mixed')

        self._email['X-XARF'] = 'PLAIN'
        self._email['Auto-Submitted'] = 'auto-generated'

        self._email.attach(MIMEText(greeting, 'plain', 'utf-8'))
        self._email.attach(
            MIMEText(xarf.to_yaml('machine_readable'), 'plain', 'utf-8')
        )

        if xarf.evidence:
            self._email.attach(MIMEText(xarf.evidence, 'plain', 'utf-8'))

    def __str__(self):
        '''
        Return the entire E-Mail flattened as a string.

        :returns: raw email
        :rtype: str
        '''
        return self._email.as_string()

    def to_string(self):
        '''
        Return the entire E-Mail flattened as a string.

        :returns: raw email
        :rtype: str
        '''
        return str(self)



'''
:copyright: (c) 2014 by abusix GmbH
:license: Apache2, see LICENSE.txt for more details.

pyxarf - easy x-arf report generation

for more information see: https://docs.abusix.com/pyxarf/

'''
import logging
import requests

from os.path import isfile
from yaml import dump as yaml_dumps
from json import dumps as json_dumps, loads as json_loads
from jsonschema.validators import Draft3Validator
from .exceptions import GeneralError, MissingParameterError, ValidationError

__version__ = '0.0.5'
__useragent__ = 'pyxarf %s' % (__version__)

class Xarf(object):
    '''
    xarf report generation class

    :param evidence: raw evidence data
    :type evidence: string
    :param reported_from: source of report
    :type reported_from: string
    :param category: category of report
    :type category: string
    :param report_type: type of report
    :type report_type: string
    :param report_id: id of report
    :type report_id: string
    :param date: date of report
    :type date: string
    :param source: source of report
    :type source: string
    :param source_type: type of report source
    :type source_type: string
    :param attachment: mime type of report attachment
    :type attachment: string
    :param schema_url: url of json schema
    :type schema_url: string
    :param schema_cache: path where to cache schemas
    :type schema_cache: string
    :param kwargs: schema dependant additional parameters
    :type kwargs: dict

    '''
    # always set our own user agent
    http_headers = {
        'User-Agent': __useragent__
    }

    # mandatory schema items
    mandatory = (
        'Reported-From',
        'Category',
        'Report-Type',
        'Report-ID',
        'Date',
        'Source',
        'Source-Type',
        'Attachment',
        'Schema-URL',
    )

    def _debug(self, *messages):
        '''
        prints debug message on stdout

        :param messages: messages to log
        :type messages: tuple

        '''
        self._logger.debug(' '.join([str(x) for x in messages]))

    def __init__(
        self,
        evidence=None,
        reported_from=None,
        category=None,
        report_type=None,
        report_id=None,
        date=None,
        source=None,
        source_type=None,
        attachment=None,
        schema_url=None,
        schema_cache=None,
        **kwargs
    ):
        '''
        handles initialization with all xarf parameters as kwargs or
        full xarf reports as dict passed to param machine_readable

        '''
        self._logger = self._setup_logging()

        self.user_agent = __useragent__
        self._required_keys = {}

        self.machine_readable = None
        self.evidence = evidence
        self.schema_url = schema_url
        self.schema_cache = schema_cache

        (self.schema, self._required_keys) = self._get_schema_and_keys(
            self._get_schema_url(self.machine_readable)
        )
        self._build_machine_readable(locals(), kwargs)

    def _setup_logging(self):
        '''
        setup basic logging configuration

        :returns: logger object
        :rtype: object

        '''
        logging.basicConfig(
            format='%(asctime)s %(name)s %(levelname)s - %(message)s'
        )
        return logging.getLogger(__name__)

    def _get_schema_and_keys(self, schema_url):
        '''
        download and convert from draft02 to draft03, also write all
        required keys to self._required_keys

        :param schema_url: url to get schema from
        :type schema_url: string

        :returns: output of :py:func:`_schema_converter`
        :rtype: tuple

        '''
        return self._schema_converter(
            self._download_schema(schema_url)
        )

    def _get_schema_url(self, machine_readable=None):
        '''
        check for schema url as parameter or in machine_readable data

        :param machine_readable: xarf machine readable part
        :type machine_readable: dict

        :returns: schema url
        :rtype: string

        :raises: :py:class:`MissingParameterError`: if schema url not defined

        '''
        if self.schema_url:
                return self.schema_url
        if machine_readable:
            if 'Schema-URL' in machine_readable:
                self.schema_url = machine_readable['Schema-URL']
                return self.schema_url
        else:
            raise MissingParameterError(
                'no schema url defined', ['schema_url']
            )

    def _build_machine_readable(self, _locals, kwargs):
        '''
        check kwargs and locals for required keys and use the given value

        :param _locals: locals of :py:func:`__init__`
        :type _locals: dict
        :param kwargs: kwargs of :py:func:`__init__`
        :type kwargs: dict

        :raises: :py:class:`MissingParameterError`: if a parameter is missing

        '''
        self.missing_parameters = []

        for key in self._required_keys:
            valid_param = key.lower().replace('-', '_')

            if valid_param in kwargs:
                self._required_keys[key] = kwargs[valid_param]
            elif valid_param in _locals:
                self._required_keys[key] = _locals[valid_param]
            else:
                self.missing_parameters.append(valid_param)
        else:
            if len(self.missing_parameters):
                raise MissingParameterError(
                    'missing required parameter(s): %s' % (
                        ', '.join(self.missing_parameters)
                    ), self.missing_parameters
                )

        self.machine_readable = self._required_keys
        self.machine_readable['User-Agent'] = __useragent__

        if not self.evidence:
            self.machine_readable['Attachment'] = 'none'

        self._debug('required schema keys', self._required_keys)

    def _download_schema(self, schema_url):
        '''
        downloads and reads schema from given url or path and returns
        it as dict

        :param schema_url: url to get schema from
        :type schema_url: string

        :returns: json schema from download or schema cache
        :rtype: dict

        :raises: :py:class:`GeneralError`: if cache access failed
        :raises: :py:class:`GeneralError`: if download of schema failed
        :raises: :py:class:`GeneralError`: if serialization failed

        '''
        schema_file = schema_url.rpartition('/')[-1]
        full_path = ''
        schema = None

        try:
            if self.schema_cache:
                full_path = '%s/%s' % (
                    self.schema_cache.rstrip('/'), schema_file
                )

                if isfile(full_path):
                    self._debug('getting schema from cache:', full_path)

                    with open(full_path, 'r') as f:
                        schema = f.read()

                    self._debug('got schema from file:', schema)
        except Exception as error:
            raise GeneralError(
                'could not get schema from cache: %s' % error
            )

        if not schema:
            try:
                self._debug('downloading schema', schema_url)
                response = requests.get(schema_url, headers=self.http_headers)
                schema = response.text

                self._debug('downloaded schema:', schema)
            except Exception as error:
                raise GeneralError(
                    'could not download schema: %s' % error
                )

        if self.schema_cache:
            with open(full_path, 'w') as f:
                f.write(schema)
        try:
            return json_loads(schema)
        except Exception as error:
            raise GeneralError(
                'could not serialize schema: %s' % error
            )

    def _validate_schema(self, schema, machine_readable):
        '''
        validates given machine_readable data against given schema
        with jsonschema draft03 validator.

        :param schema: json schema to check against
        :type schema: dict
        :param machine_readable: xarf machine readable part
        :type machine_readable: dict

        :raises: :py:class:`ValidationError`: if validation fails

        '''
        errors = []
        validator = Draft3Validator(schema)
        result = validator.iter_errors(machine_readable)

        for error in result:
            msg = '%s %s' % (error.path[0], error.message)
            errors.append(msg)

        if len(errors):
            raise ValidationError(
                ', '.join(errors)
            )

    def _schema_converter(self, schema):
        '''
        converts given schema from draft02 to draft03 as jsonschema
        does not support draft02.

        :param schema: schema to validate
        :type schema: dict

        :returns: converted schema and required keys
        :rtype: tuple

        '''

        required_keys = {}
        self._debug('before conversion:', json_dumps(schema))

        if 'properties' in schema:
            props = schema['properties']

            for item in props:
                if item == 'User-Agent':
                    continue
                if 'requires' in props[item]:
                    props[item]['dependencies'] = props[item]['requires']
                    del props[item]['requires']
                if 'optional' not in props[item]:
                    props[item]['required'] = True
                    required_keys[str(item)] = ''
                else:
                    del props[item]['optional']

        self._debug('after conversion:', json_dumps(schema))
        return (schema, required_keys)

    def _get_validated_machine_readable(self):
        '''
        validates and returns the machine readable parts

        :returns: validated machine readable data
        :rtype: dict

        '''
        self._debug(self.machine_readable)
        self._validate_schema(
                self.schema,
                self.machine_readable
            )
        return self.machine_readable

    def __str__(self):
        '''
        method for convinient usage

        :returns: output of :py:func:`to_json`
        :rtype: json

        '''
        return self.to_json()

    def to_json(self, part=None):
        '''
        returns data as json

        :param part: X-ARF object part machine_readable or evidence
        :type part: str
        :returns: json dump of xarf report
        :rtype: json

        '''
        return json_dumps(self.get_report_obj(part))

    def to_yaml(self, part=None):
        '''
        returns data as yaml

        :param part: X-ARF object part machine_readable or evidence
        :type part: str
        :returns: yaml dump of xarf report
        :rtype: yaml

        '''
        return yaml_dumps(
            self.get_report_obj(part), default_flow_style=False
        )

    def get_report_obj(self, part=None):
        '''
        returns the validated data with meta information as python dict

        :param part: X-ARF object part machine_readable or evidence
        :type part: str
        :returns: xarf report as list with both parts
        :rtype: list
        '''

        if part:
            if part == 'machine_readable':
                return self._get_validated_machine_readable()
            elif part == 'evidence':
                return self.evidence

        return {
            'machine_readable': self._get_validated_machine_readable(),
            'evidence': self.evidence,
        }

    def add_evidence(self, evidence):
        '''
        add evidence to xarf report

        :param evidence: raw evidence data
        :type evidence: string

        '''
        self.evidence = evidence

    @classmethod
    def from_machine_readable(cls, machine_readable, evidence=None):
        '''
        xarf factory to create xarf object from dict.

        :param machine_readable: xarf machine readable part
        :type machine_readable: dict
        :param evidence: raw evidence data
        :type evidence: string

        :returns: instance of :py:class:`Xarf`
        :rtype: instance

        '''
        data = {}
        for key in machine_readable:
            valid_key = key.lower().replace('-', '_')
            data[valid_key] = machine_readable[key]
        data['evidence'] = evidence
        return cls(**data)



'''
:copyright: (c) 2014 by abusix GmbH
:license: Apache2, see LICENSE.txt for more details.

some custom exception handlers

'''

class GeneralError(Exception):
    '''
    default exception class

    '''
    pass

class ValidationError(Exception):
    '''
    gets raised on validation exceptions

    '''
    pass

class MissingParameterError(Exception):
    '''
    gets raised when a parameter is missing

    '''
    def __init__(self, message, missing_parameters):
        '''
        subclassing exception class to pass the missing parameters
        through the exception object to the calling application

        :param message: original error message
        :type message: str
        :param missing_parameters: list of missing parameters
        :type missing_parameters: list

        '''
        Exception.__init__(self, message)
        self.missing_parameters = missing_parameters


import sys

from os import path
from setuptools import setup, find_packages

__version__ = None
here = path.abspath(path.dirname(__file__))
name = 'pyxarf'

with open(path.join(here, 'README.md')) as f:
    long_description = f.read()

with open(path.join(here, 'version.txt')) as f:
    __version__ = f.readline().strip()

with open(path.join(here, 'requirements.txt')) as f:
    requires = f.readlines()

setup(
    author='abusix',
    author_email='info@abusix.com',
    description='pyxarf - easy x-arf report generation',
    long_description=long_description,
    long_description_content_type="text/markdown",
    name=name,
    packages=find_packages(),
    install_requires=requires,
    python_requires='>=2.7',
    url='http://xarf.org/',
    project_urls={
        'Source': 'https://github.com/xarf/python-xarf',
        'Company': 'https://www.abusix.com/'
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Customer Service',
        'Intended Audience :: System Administrators',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Security',
        'License :: OSI Approved :: Apache Software License'

    ],
    keywords = ['keyword', 'search', 'purepython', 'aho-corasick', 'ahocorasick', 'abusix'],
    license='Apache Software License',
    version=__version__,
    scripts=['scripts/xarfutil.py'],
)
